@@ -92,7 +92,7 @@ int pam_dp_send_req(struct pam_auth_req *preq, int timeout)
      * dp connection code is actually able to establish a connection.
      */
     res = sss_dp_get_domain_conn(preq->cctx->rctx,
-                                 preq->domain->name, &be_conn);
+                                 preq->domain->conn_name, &be_conn);
     if (res != EOK) {
         DEBUG(1, ("The Data Provider connection for %s is not available!"
                   " This maybe a bug, it shouldn't happen!\n", preq->domain));
