@@ -195,6 +195,7 @@ static enum sss_status sss_cli_recv_rep(enum sss_cli_command cmd,
     uint32_t header[4];
     size_t datarecv;
     uint8_t *buf = NULL;
+    bool pollhup = false;
     int len;
     int ret;
 
@@ -235,7 +236,10 @@ static enum sss_status sss_cli_recv_rep(enum sss_cli_command cmd,
             *errnop = ETIME;
             break;
         case 1:
-            if (pfd.revents & (POLLERR | POLLHUP | POLLNVAL)) {
+            if (pfd.revents & (POLLHUP)) {
+                pollhup = true;
+            }
+            if (pfd.revents & (POLLERR | POLLNVAL)) {
                 *errnop = EPIPE;
             }
             if (!(pfd.revents & POLLIN)) {
@@ -322,6 +326,10 @@ static enum sss_status sss_cli_recv_rep(enum sss_cli_command cmd,
         }
     }
 
+    if (pollhup) {
+        sss_cli_close_socket();
+    }
+
     *_len = len;
     *_buf = buf;
 
@@ -786,6 +794,7 @@ errno_t check_server_cred(int sockfd)
 #endif
     return 0;
 }
+
 int sss_pam_make_request(enum sss_cli_command cmd,
                       struct sss_cli_req_data *rd,
                       uint8_t **repbuf, size_t *replen,
@@ -865,6 +874,18 @@ out:
     return ret;
 }
 
+void sss_pam_close_fd(void)
+{
+    sss_pam_lock();
+
+    if (sss_cli_sd != -1) {
+        close(sss_cli_sd);
+        sss_cli_sd = -1;
+    }
+
+    sss_pam_unlock();
+}
+
 int sss_sudo_make_request(enum sss_cli_command cmd,
                           struct sss_cli_req_data *rd,
                           uint8_t **repbuf, size_t *replen,
