@@ -60,10 +60,8 @@ START_TEST(test_encrypt_decrypt)
     int ret;
     int expected;
 
-#ifdef HAVE_NSS
+#if defined(HAVE_NSS) || defined(HAVE_LIBCRYPTO)
     expected = EOK;
-#elif HAVE_LIBCRYPTO
-    expected = ENOSYS;
 #else
 #error Unknown crypto back end
 #endif
@@ -91,6 +89,75 @@ START_TEST(test_encrypt_decrypt)
 }
 END_TEST
 
+START_TEST(test_hmac_sha1)
+{
+    const char *message = "test message";
+    const char *keys[] = {
+        "short",
+        "proper6789012345678901234567890123456789012345678901234567890123",
+        "longlonglonglonglonglonglonglonglonglonglonglonglonglonglonglonglong",
+        NULL };
+    const char *results[] = {
+        "\x2b\x27\x53\x07\x17\xd8\xc0\x8f\x97\x27\xdd\xb3\xec\x41\xd8\xa3\x94\x97\xaa\x35",
+        "\x37\xe7\x0a\x6f\x71\x0b\xa9\x93\x81\x53\x8f\x5c\x06\x83\x44\x2f\xc9\x41\xe3\xed",
+        "\xbd\x99\xa7\x7f\xfc\x5e\xde\x04\x32\x7f\x7b\x71\x4d\xc0\x3f\x51\x2d\x25\x01\x28",
+        NULL };
+    unsigned char out[SSS_SHA1_LENGTH];
+    int ret, expected;
+    int i;
+
+#if defined(HAVE_NSS) || defined(HAVE_LIBCRYPTO)
+    expected = EOK;
+#else
+#error Unknown crypto back end
+#endif
+
+    for (i = 0; keys[i]; i++) {
+        ret = sss_hmac_sha1((const unsigned char *)keys[i], strlen(keys[i]),
+                            (const unsigned char *)message, strlen(message),
+                            out);
+        fail_if(ret != expected);
+        fail_if(ret == EOK && memcmp(out, results[i], SSS_SHA1_LENGTH) != 0);
+    }
+}
+END_TEST
+
+START_TEST(test_base64_encode)
+{
+    const unsigned char obfbuf[] = "test";
+    const char expected[] = "dGVzdA==";
+    char *obfpwd = NULL;
+
+    test_ctx = talloc_new(NULL);
+    fail_if(test_ctx == NULL);
+    /* Base64 encode the buffer */
+    obfpwd = sss_base64_encode(test_ctx, obfbuf, strlen((const char*)obfbuf));
+    fail_if(obfpwd == NULL);
+    fail_if(strcmp(obfpwd,expected) != 0);
+
+    talloc_free(test_ctx);
+}
+END_TEST
+
+START_TEST(test_base64_decode)
+{
+    unsigned char *obfbuf = NULL;
+    size_t obflen;
+    const char b64encoded[] = "dGVzdA==";
+    const unsigned char expected[] = "test";
+
+    test_ctx = talloc_new(NULL);
+    fail_if(test_ctx == NULL);
+    /* Base64 decode the buffer */
+    obfbuf = sss_base64_decode(test_ctx, b64encoded, &obflen);
+    fail_if(!obfbuf);
+    fail_if(obflen != strlen((const char*)expected));
+    fail_if(memcmp(obfbuf, expected, obflen) != 0);
+
+    talloc_free(test_ctx);
+}
+END_TEST
+
 Suite *crypto_suite(void)
 {
     Suite *s = suite_create("sss_crypto");
@@ -102,6 +169,9 @@ Suite *crypto_suite(void)
     tcase_add_test(tc, test_nss_init);
 #endif
     tcase_add_test(tc, test_encrypt_decrypt);
+    tcase_add_test(tc, test_hmac_sha1);
+    tcase_add_test(tc, test_base64_encode);
+    tcase_add_test(tc, test_base64_decode);
     /* Add all test cases to the test suite */
     suite_add_tcase(s, tc);
 
@@ -114,11 +184,10 @@ int main(int argc, const char *argv[])
     int opt;
     poptContext pc;
     int number_failed;
-    int debug = 0;
 
     struct poptOption long_options[] = {
         POPT_AUTOHELP
-        { "debug-level", 'd', POPT_ARG_INT, &debug, 0, "Set debug level", NULL },
+        { "debug-level", 'd', POPT_ARG_INT, &debug_level, 0, "Set debug level", NULL },
         POPT_TABLEEND
     };
 
