@@ -963,3 +963,30 @@ sss_krb5_residual_check_type(const char *full_location,
 
     return sss_krb5_residual_by_type(full_location, type);
 }
+
+#ifdef HAVE_KRB5_SET_TRACE_CALLBACK
+static void
+sss_child_krb5_trace_cb(krb5_context context,
+                        const struct krb5_trace_info *info, void *data)
+{
+    if (info == NULL) {
+        /* Null info means destroy the callback data. */
+        return;
+    }
+
+    DEBUG(SSSDBG_TRACE_ALL, ("%s\n", info->message));
+}
+
+errno_t
+sss_child_set_krb5_tracing(krb5_context ctx)
+{
+    return krb5_set_trace_callback(ctx, sss_child_krb5_trace_cb, NULL);
+}
+#else /* HAVE_KRB5_SET_TRACE_CALLBACK */
+errno_t
+sss_child_set_krb5_tracing(krb5_context ctx)
+{
+    DEBUG(SSSDBG_CONF_SETTINGS, ("krb5 tracing is not available\n"));
+    return 0;
+}
+#endif /* HAVE_KRB5_SET_TRACE_CALLBACK */
