@@ -34,6 +34,9 @@
 
 #include "util/util.h"
 
+#define KRB5_CHILD_LOG_FILE     "krb5_child"
+#define LDAP_CHILD_LOG_FILE     "ldap_child"
+
 /* MIT Kerberos has the same hardcoded warning interval of 7 days. Due to the
  * fact that using the expiration time of a Kerberos password with LDAP
  * authentication is presumably a rare case a separate config option is not
@@ -164,4 +167,7 @@ typedef krb5_ticket_times sss_krb5_ticket_times;
 typedef krb5_times sss_krb5_ticket_times;
 #endif
 
+/* Redirect libkrb5 tracing towards our DEBUG statements */
+errno_t sss_child_set_krb5_tracing(krb5_context ctx);
+
 #endif /* __SSS_KRB5_H__ */
